@@ -62,10 +62,12 @@ module CarrierWave
       begin
         require "rmagick"
       rescue LoadError
-        require "RMagick"
-      rescue LoadError => e
-        e.message << " (You may need to install the rmagick gem)"
-        raise e
+        begin
+          require "RMagick"
+        rescue LoadError => e
+          e.message << " (You may need to install the rmagick gem)"
+          raise e
+        end
       end
 
       prepend Module.new {
@@ -109,7 +111,7 @@ module CarrierWave
     #
     # === Parameters
     #
-    # [format (#to_s)] an abreviation of the format
+    # [format (#to_s)] an abbreviation of the format
     #
     # === Yields
     #
@@ -228,13 +230,7 @@ module CarrierWave
       height = dimension_from height
       manipulate! do |img|
         img.resize_to_fit!(width, height)
-        new_img = ::Magick::Image.new(width, height) { self.background_color = background == :transparent ? 'rgba(255,255,255,0)' : background.to_s }
-        if background == :transparent
-          filled = new_img.matte_floodfill(1, 1)
-        else
-          filled = new_img.color_floodfill(1, 1, ::Magick::Pixel.from_color(background))
-        end
-        destroy_image(new_img)
+        filled = ::Magick::Image.new(width, height) { |image| image.background_color = background == :transparent ? 'rgba(255,255,255,0)' : background.to_s }
         filled.composite!(img, gravity, ::Magick::OverCompositeOp)
         destroy_image(img)
         filled = yield(filled) if block_given?
@@ -363,15 +359,15 @@ module CarrierWave
       if options[:format] || @format
         frames.write("#{options[:format] || @format}:#{current_path}", &write_block)
         move_to = current_path.chomp(File.extname(current_path)) + ".#{options[:format] || @format}"
-        file.content_type = ::MIME::Types.type_for(move_to).first.to_s
+        file.content_type = Marcel::Magic.by_path(move_to).try(:type)
         file.move_to(move_to, permissions, directory_permissions)
       else
         frames.write(current_path, &write_block)
       end
 
       destroy_image(frames)
-    rescue ::Magick::ImageMagickError => e
-      raise CarrierWave::ProcessingError, I18n.translate(:"errors.messages.rmagick_processing_error", :e => e)
+    rescue ::Magick::ImageMagickError
+      raise CarrierWave::ProcessingError, I18n.translate(:"errors.messages.processing_error")
     end
 
   private
