@@ -114,15 +114,15 @@ describe CarrierWave::Mount do
       end
 
       context "defined processors inheritance works" do
-        it { expect(uploader_1.processors).to eq([[:rotate, [], nil]]) }
+        it { expect(uploader_1.processors).to eq([[:rotate, [], nil, nil]]) }
 
-        it { expect(uploader_2.processors).to eq([[:rotate, [], nil], [:shrink, [], nil]]) }
+        it { expect(uploader_2.processors).to eq([[:rotate, [], nil, nil], [:shrink, [], nil, nil]]) }
 
-        it { expect(uploader_1.versions[:thumb].processors).to eq([[:compress, [], nil]]) }
+        it { expect(uploader_1.new.versions[:thumb].processors).to eq([[:compress, [], nil, nil]]) }
 
-        it { expect(uploader_2.versions[:thumb].processors).to eq([[:compress, [], nil]]) }
+        it { expect(uploader_2.new.versions[:thumb].processors).to eq([[:compress, [], nil, nil]]) }
 
-        it { expect(uploader_2.versions[:secret].processors).to eq([[:encrypt, [], nil]]) }
+        it { expect(uploader_2.new.versions[:secret].processors).to eq([[:encrypt, [], nil, nil]]) }
       end
     end
 
@@ -181,6 +181,11 @@ describe CarrierWave::Mount do
         it "copies files into the cache directory" do
           expect(instance.images[0].current_path).to match(/^#{public_path('uploads/tmp')}/)
         end
+
+        it "marks the uploader as staged" do
+          expect(instance.images[0].staged).to be true
+          expect(instance.images[1].staged).to be true
+        end
       end
 
       it "does nothing when nil is assigned" do
@@ -190,51 +195,185 @@ describe CarrierWave::Mount do
 
       it "does nothing when an empty string is assigned" do
         expect(instance).not_to receive(:write_uploader)
-
-        instance.images = [test_file_stub]
+        instance.images = ''
       end
 
-      context "fails silently if the images fails a white list integrity check" do
+      it "accepts another uploader instances" do
+        instance.images = [test_file_stub, old_image_stub]
+        instance.store_images!
+        another = klass.new
+        another.images = instance.images
+        expect(another.images).to all(be_an_instance_of(uploader))
+        expect(another.images.map(&:cached?)).to all(be true)
+      end
+
+      context "if the images fails an allowlist integrity check" do
         before do
           uploader.class_eval do
-            def extension_whitelist
+            def extension_allowlist
               %w(txt)
             end
           end
-
-          instance.images = [text_file_stub, test_file_stub]
         end
 
-        it { expect(instance.images).to be_empty }
+        it "fails silently" do
+          expect { instance.images = [test_file_stub] }.not_to raise_error
+        end
+
+        it "keeps files which passed the check" do
+          instance.images = [test_file_stub, text_file_stub]
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt']
+        end
       end
 
-      describe "fails silently if the images fails a black list integrity check" do
+      describe "if the images fails a denylist integrity check" do
         before do
           uploader.class_eval do
-            def extension_blacklist
-              %w(jpg)
+            def extension_allowlist
+              %w(png txt)
             end
           end
-
-          instance.images = [text_file_stub, test_file_stub]
         end
 
-        it { expect(instance.images).to be_empty }
+        it "fails silently" do
+          expect { instance.images = [test_file_stub] }.not_to raise_error
+        end
+
+        it "keeps files which passed the check" do
+          instance.images = [test_file_stub, text_file_stub]
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt']
+        end
       end
 
-      describe "fails silently if the images fails to be processed" do
+      describe "if the images fails to be processed" do
         before do
           uploader.class_eval do
             process :monkey
             def monkey
-              raise CarrierWave::ProcessingError, "Ohh noez!"
+              raise CarrierWave::ProcessingError, "Ohh noez!" if file.path =~ /test\.jpg/
             end
           end
-
-          instance.images = [test_file_stub]
         end
 
-        it { expect(instance.images).to be_empty }
+        it "fails silently" do
+          expect { instance.images = [test_file_stub] }.not_to raise_error
+        end
+
+        it "keeps files which was processed successfully" do
+          instance.images = [test_file_stub, text_file_stub]
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt']
+        end
+      end
+
+      describe "when caching files of the same filename" do
+        before { FileUtils.cp(file_path('bork.json'), tmp_path('bork.txt')) }
+        after { FileUtils.rm(tmp_path('bork.txt')) }
+
+        it "accepts them without confusion" do
+          instance.images = [text_file_stub, File.open(tmp_path('bork.txt'))]
+          expect(instance.images[0].cache_path).not_to eq instance.images[1].cache_path
+          expect(instance.images[0].read).not_to eq instance.images[1].read
+        end
+      end
+
+      describe "with cached files" do
+        before do
+          instance.images = [text_file_stub, test_file_stub]
+        end
+        let(:cache_names) { instance.images.map(&:cache_name) }
+
+        it "accepts cache name and retrieves from cache" do
+          instance.images = [cache_names[1]]
+          expect(instance.images.map { |u| u.file.filename }).to eq ['test.jpg']
+        end
+
+        context "when adding a file which has the same filename with the exsting one" do
+          before { FileUtils.cp(file_path('bork.json'), tmp_path('bork.txt')) }
+          after { FileUtils.rm(tmp_path('bork.txt')) }
+
+          it "accepts it without confusion" do
+            instance.images = [instance.images[0].cache_name, File.open(tmp_path('bork.txt'))]
+            expect(instance.images[0].cache_path).not_to eq instance.images[1].cache_path
+            expect(instance.images[0].read).not_to eq instance.images[1].read
+          end
+        end
+      end
+
+      describe "with stored files" do
+        before do
+          instance.images = [text_file_stub, test_file_stub]
+          instance.store_images!
+        end
+        let(:identifiers) { instance.images.map(&:identifier) }
+
+        it "writes over a previously stored file" do
+          instance.images = [old_image_stub]
+          instance.store_images!
+          expect(instance.images.map(&:identifier)).to eq ['old.jpeg']
+        end
+
+        it "preserves existing image of given identifier" do
+          instance.images = [identifiers[0], old_image_stub]
+          instance.store_images!
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt', 'old.jpeg']
+        end
+
+        it "reorders existing image" do
+          instance.images = identifiers.reverse
+          instance.store_images!
+          expect(instance.images.map(&:identifier)).to eq ['test.jpg', 'bork.txt']
+        end
+
+        it "allows uploading and reordering at once" do
+          instance.images = [identifiers[1], old_image_stub, identifiers[0]]
+          instance.store_images!
+          expect(instance.images.map(&:identifier)).to eq ['test.jpg', 'old.jpeg', 'bork.txt']
+        end
+
+        it "allows repeating the same identifiers" do
+          instance.images = ['bork.txt', 'test.jpg', 'bork.txt']
+          instance.store_images!
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt', 'test.jpg', 'bork.txt']
+        end
+
+        it "removes image which is unused" do
+          @image_paths = instance.images.map(&:current_path)
+          instance.images = [identifiers[0]]
+          instance.store_images!
+          instance.send(:_mounter, :images).remove_previous
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt']
+          expect(File.exist?(@image_paths[0])).to be_truthy
+          expect(File.exist?(@image_paths[1])).to be_falsey
+        end
+
+        it "ignores unknown identifier" do
+          instance.images = ['unknown.txt']
+          expect { instance.store_images! }.not_to raise_error
+          expect(instance.images.map(&:identifier)).to be_empty
+        end
+
+        it "allows deleting all files" do
+          instance.images = []
+          expect(instance.images.map(&:identifier)).to be_empty
+        end
+
+        it "allows assignment of uploader instances" do
+          instance.images = [instance.images[0]]
+          expect(instance.images.map(&:identifier)).to eq ['bork.txt']
+        end
+
+        context "when adding a file which has the same filename with the exsting one" do
+          before { FileUtils.cp(file_path('bork.json'), tmp_path('bork.txt')) }
+          after { FileUtils.rm(tmp_path('bork.txt')) }
+
+          it "renames the latter file to avoid filename duplication" do
+            instance.images = ['bork.txt', File.open(tmp_path('bork.txt'))]
+            instance.write_images_identifier
+            instance.store_images!
+            expect(instance.images.map(&:identifier)).to eq ['bork.txt', 'bork(2).txt']
+            expect(instance.images[0].read).not_to eq instance.images[1].read
+          end
+        end
       end
     end
 
@@ -342,33 +481,66 @@ describe CarrierWave::Mount do
         CarrierWave::SanitizedFile.new(test_file_stub).copy_to(public_path('uploads/tmp/1369894322-123-0123-1234/test.jpg'))
       end
 
-      before { instance.images_cache = images_cache }
-
       context "does nothing when nil is assigned" do
-        let(:images_cache) { nil }
+        before { instance.images_cache = nil }
 
         it { expect(instance.images).to be_empty }
       end
 
       context "does nothing when an empty string is assigned" do
-        let(:images_cache) { '' }
+        before { instance.images_cache = '' }
 
         it { expect(instance.images).to be_empty }
       end
 
       context "retrieve from cache when a cache name is assigned" do
-        let(:images_cache) { ['1369894322-123-0123-1234/test.jpg'].to_json }
+        before { instance.images_cache = ['1369894322-123-0123-1234/test.jpg'].to_json }
 
         it { expect(instance.images[0].current_path).to eq(public_path('uploads/tmp/1369894322-123-0123-1234/test.jpg')) }
+
+        it "marks the uploader as staged" do
+          expect(instance.images[0].staged).to be true
+        end
+      end
+
+      context "writes over a previously stored file" do
+        before do
+          instance.images = [test_file_stub]
+          instance.store_images!
+          instance.images_cache = ['1369894322-123-0123-1234/monkey.jpg'].to_json
+        end
+
+        it { expect(instance.images[0].current_path).to match(/monkey.jpg$/) }
       end
 
       context "doesn't write over a previously assigned file" do
-        let(:images_cache) { ['1369894322-123-0123-1234/monkey.jpg'].to_json }
-
-        before { instance.images = [test_file_stub] }
+        before do
+          instance.images = [test_file_stub]
+          instance.images_cache = ['1369894322-123-0123-1234/monkey.jpg'].to_json
+        end
 
         it { expect(instance.images[0].current_path).to match(/test.jpg$/) }
       end
+
+      context "when valid and invalid cache names are assigned" do
+        before { instance.images_cache = ['1369894322-123-0123-1234/test.jpg', 'invalid'].to_json }
+
+        it "retrieves valid file only from cache" do
+          expect(instance.images.map(&:cache_name)).to eq(['1369894322-123-0123-1234/test.jpg'])
+        end
+      end
+
+      context "when an empty string is assigned" do
+        before do
+          instance.images = [test_file_stub]
+          instance.store_images!
+          instance.images_cache = [''].to_json
+        end
+
+        it "does not write over a previously stored file" do
+          expect(instance.images[0].current_path).to match(/test.jpg$/)
+        end
+      end
     end
 
     describe "#remote_images_urls" do
@@ -392,6 +564,7 @@ describe CarrierWave::Mount do
 
       before do
         stub_request(:get, "http://www.example.com/#{test_file_name}").to_return(body: File.read(test_file_stub))
+        stub_request(:get, "http://www.example.com/test.txt").to_return(status: 404)
         instance.remote_images_urls = remote_images_url
       end
 
@@ -413,9 +586,27 @@ describe CarrierWave::Mount do
         let(:remote_images_url) { ["http://www.example.com/test.jpg"] }
 
         it { is_expected.to match(/test.jpg$/) }
+
+        it "marks the uploader as staged" do
+          expect(instance.images[0].staged).to be true
+        end
       end
 
-      context "writes over a previously assigned file" do
+      context "writes over a previously stored file" do
+        subject { images[0].current_path }
+
+        let(:remote_images_url) { ["http://www.example.com/test.jpg"] }
+
+        before do
+          instance.images = [stub_file("portrait.jpg")]
+          instance.store_images!
+          instance.remote_images_urls = remote_images_url
+        end
+
+        it { is_expected.to match(/test.jpg$/) }
+      end
+
+      context "does not write over a previously assigned file" do
         subject { images[0].current_path }
 
         let(:remote_images_url) { ["http://www.example.com/test.jpg"] }
@@ -425,7 +616,42 @@ describe CarrierWave::Mount do
           instance.remote_images_urls = remote_images_url
         end
 
-        it { is_expected.to match(/test.jpg$/) }
+        it { is_expected.to match(/portrait.jpg$/) }
+      end
+
+      context "when an empty string is assigned" do
+        subject { images[0].current_path }
+
+        let(:remote_images_url) { [""] }
+
+        before do
+          instance.images = [stub_file("portrait.jpg")]
+          instance.store_images!
+          instance.remote_images_urls = remote_images_url
+        end
+
+        it "does not write over a previously stored file" do
+          is_expected.to match(/portrait.jpg$/)
+        end
+      end
+
+      context "if a file fails to be downloaded" do
+        let(:remote_images_url) { ["http://www.example.com/test.txt", "http://www.example.com/test.jpg"] }
+
+        it "keeps files which was downloaded successfully" do
+          expect(instance.images.map(&:identifier)).to eq ['test.jpg']
+        end
+      end
+
+      context "clears the unsaved remote urls when nil is assigned" do
+        subject { instance.remote_images_urls }
+        let(:remote_images_url) { ['invalid'] }
+
+        before do
+          instance.remote_images_urls = nil
+        end
+
+        it { is_expected.to be_empty }
       end
     end
 
@@ -450,21 +676,10 @@ describe CarrierWave::Mount do
         end
 
         it { expect(instance.images[0].current_path).to eq(public_path("uploads/#{test_file_name}")) }
-      end
 
-      context "removes an uploaded file when remove_images is true" do
-        let(:images) { [test_file_stub] }
-
-        before do
-          instance.images = images
-          @image_path = instance.images[0].current_path.dup
-          instance.remove_images = true
-          instance.store_images!
+        it "marks the uploader as unstaged" do
+          expect(instance.images[0].staged).to be false
         end
-
-        it { expect(instance.images).to be_empty }
-
-        it { expect(File.exist?(@image_path)).to be_falsey }
       end
     end
 
@@ -524,36 +739,42 @@ describe CarrierWave::Mount do
         it { is_expected.to be_falsey }
       end
 
-      context "when value is 0" do
+      context "when value is '0'" do
         let(:remove_images) { "0" }
 
         it { is_expected.to be_falsey }
       end
 
-      context "when value is false" do
+      context "when value is 'false'" do
         let(:remove_images) { 'false' }
 
         it { is_expected.to be_falsey }
       end
+
+      context "when value is 0" do
+        let(:remove_images) { 0 }
+
+        it { is_expected.to be_falsey }
+      end
     end
 
-    describe '#images_integrity_error' do
-      subject(:images_integrity_error) { instance.images_integrity_error }
+    describe '#images_integrity_errors' do
+      subject(:images_integrity_errors) { instance.images_integrity_errors }
 
       describe "default behaviour" do
-        it { is_expected.to be_nil }
+        it { is_expected.to be_empty }
       end
 
       context "when a file is cached" do
         before { instance.images = test_file_stub }
 
-        it { is_expected.to be_nil }
+        it { is_expected.to be_empty }
       end
 
       describe "when an integrity check fails" do
         before do
           uploader.class_eval do
-            def extension_whitelist
+            def extension_allowlist
               %w(txt)
             end
           end
@@ -562,10 +783,10 @@ describe CarrierWave::Mount do
         context "when file is cached" do
           before { instance.images = [test_file_stub] }
 
-          it { is_expected.to be_an_instance_of(CarrierWave::IntegrityError) }
+          it { is_expected.to include(a_kind_of(CarrierWave::IntegrityError)) }
 
           it "has an error message" do
-            expect(images_integrity_error.message.lines.grep(/^You are not allowed to upload/)).to be_truthy
+            expect(images_integrity_errors[0].message.lines.grep(/^You are not allowed to upload/)).to be_truthy
           end
         end
 
@@ -575,39 +796,39 @@ describe CarrierWave::Mount do
             instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]
           end
 
-          it { is_expected.to be_an_instance_of(CarrierWave::IntegrityError) }
+          it { is_expected.to include(a_kind_of(CarrierWave::IntegrityError)) }
 
           it "has an error message" do
-            expect(images_integrity_error.message.lines.grep(/^You are not allowed to upload/)).to be_truthy
+            expect(images_integrity_errors[0].message.lines.grep(/^You are not allowed to upload/)).to be_truthy
           end
         end
 
-        context "when file is assigned and remote_iamges_url is blank" do
+        context "when file is assigned and remote_images_url is blank" do
           before do
             instance.images = [test_file_stub]
             instance.remote_images_urls = ""
           end
 
-          it { is_expected.to be_an_instance_of(CarrierWave::IntegrityError) }
+          it { is_expected.to include(a_kind_of(CarrierWave::IntegrityError)) }
 
           it "has an error message" do
-            expect(images_integrity_error.message.lines.grep(/^You are not allowed to upload/)).to be_truthy
+            expect(images_integrity_errors[0].message.lines.grep(/^You are not allowed to upload/)).to be_truthy
           end
         end
       end
     end
 
-    describe '#images_processing_error' do
-      subject(:images_processing_error) { instance.images_processing_error }
+    describe '#images_processing_errors' do
+      subject(:images_processing_errors) { instance.images_processing_errors }
 
       describe "default behavior" do
-        it { is_expected.to be_nil }
+        it { is_expected.to be_empty }
       end
 
       context "when file is cached" do
         before { instance.images = [test_file_stub] }
 
-        it { is_expected.to be_nil }
+        it { is_expected.to be_empty }
       end
 
       describe "when an processing error occurs" do
@@ -623,7 +844,7 @@ describe CarrierWave::Mount do
         context "when file is cached" do
           before { instance.images = [test_file_stub] }
 
-          it { is_expected.to be_an_instance_of(CarrierWave::ProcessingError) }
+          it { is_expected.to include(a_kind_of(CarrierWave::ProcessingError)) }
         end
 
         context "when file was downloaded" do
@@ -632,13 +853,13 @@ describe CarrierWave::Mount do
             instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]
           end
 
-          it { is_expected.to be_an_instance_of(CarrierWave::ProcessingError) }
+          it { is_expected.to include(a_kind_of(CarrierWave::ProcessingError)) }
         end
       end
     end
 
-    describe '#images_download_error' do
-      subject(:images_download_error) { instance.images_download_error }
+    describe '#images_download_errors' do
+      subject(:images_download_errors) { instance.images_download_errors }
 
       before do
         stub_request(:get, "http://www.example.com/#{test_file_name}").to_return(body: File.read(test_file_stub))
@@ -646,19 +867,19 @@ describe CarrierWave::Mount do
       end
 
       describe "default behaviour" do
-        it { expect(instance.images_download_error).to be_nil }
+        it { expect(instance.images_download_errors).to be_empty }
       end
 
       context "when file download was successful" do
         before { instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"] }
 
-        it { is_expected.to be_nil }
+        it { is_expected.to be_empty }
       end
 
       context "when file couldn't be found" do
         before { instance.remote_images_urls = ["http://www.example.com/missing.jpg"] }
 
-        it { is_expected.to be_an_instance_of(CarrierWave::DownloadError) }
+        it { is_expected.to include(a_kind_of(CarrierWave::DownloadError)) }
       end
     end
 
@@ -666,8 +887,8 @@ describe CarrierWave::Mount do
       after { instance.write_images_identifier }
 
       it "writes to the column" do
-        expect(instance).to receive(:write_uploader).with(:images, [test_file_name]).at_least(:once)
         instance.images = [test_file_stub]
+        expect(instance).to receive(:write_uploader).with(:images, [test_file_name]).at_least(:once)
         instance.write_images_identifier
       end
 
@@ -678,7 +899,11 @@ describe CarrierWave::Mount do
           instance.remove_images = true
         end
 
-        it "removes from the column when remove_images is true" do
+        it "clears existing uploaders" do
+          expect(instance.images).to be_empty
+        end
+
+        it "removes from the column" do
           expect(instance).to receive(:write_uploader).with(:images, nil)
         end
       end
@@ -785,6 +1010,10 @@ describe CarrierWave::Mount do
       it "applies any custom modifications to the uploader class" do
         expect(uploader.new).not_to respond_to(:fish)
       end
+
+      it "should give the generated class a stable name" do
+        expect(first_image.class).to eq klass.const_get :CarrierWaveImagesUploader
+      end
     end
   end
 
@@ -799,7 +1028,7 @@ describe CarrierWave::Mount do
     let(:uploader) do
       Class.new(CarrierWave::Uploader::Base).tap do |u|
         u.class_eval do
-          def extension_whitelist
+          def extension_allowlist
             %w(txt)
           end
         end
@@ -807,15 +1036,15 @@ describe CarrierWave::Mount do
     end
 
     context "when a cached image fails an integrity check" do
-      it { expect(running { instance.images = [test_file_stub] }).to raise_error(CarrierWave::IntegrityError) }
+      it { expect { instance.images = [test_file_stub] }.to raise_error(CarrierWave::IntegrityError) }
     end
 
-    context "when a downloaded image fails an integity check" do
+    context "when a downloaded image fails an integrity check" do
       before do
         stub_request(:get, "http://www.example.com/#{test_file_name}").to_return(body: test_file_stub)
       end
 
-      it { expect(running {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}).to raise_error(CarrierWave::IntegrityError) }
+      it { expect {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}.to raise_error(CarrierWave::IntegrityError) }
     end
   end
 
@@ -839,15 +1068,15 @@ describe CarrierWave::Mount do
     end
 
     context "when a cached image fails an integrity check" do
-      it { expect(running { instance.images = [test_file_stub] }).to raise_error(CarrierWave::ProcessingError) }
+      it { expect { instance.images = [test_file_stub] }.to raise_error(CarrierWave::ProcessingError) }
     end
 
-    context "when a downloaded image fails an integity check" do
+    context "when a downloaded image fails an integrity check" do
       before do
         stub_request(:get, "http://www.example.com/#{test_file_name}").to_return(body: test_file_stub)
       end
 
-      it { expect(running {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}).to raise_error(CarrierWave::ProcessingError) }
+      it { expect {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}.to raise_error(CarrierWave::ProcessingError) }
     end
   end
 
@@ -863,14 +1092,14 @@ describe CarrierWave::Mount do
 
     before do
       uploader.class_eval do
-        def download! uri, headers = {}
+        def download!(uri, headers = {})
           raise CarrierWave::DownloadError
         end
       end
     end
 
     context "when the image fail to be processed" do
-      it { expect(running {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}).to raise_error(CarrierWave::DownloadError) }
+      it { expect {instance.remote_images_urls = ["http://www.example.com/#{test_file_name}"]}.to raise_error(CarrierWave::DownloadError) }
     end
   end
 
@@ -896,8 +1125,8 @@ describe CarrierWave::Mount do
 
     describe '#write_images_identifier' do
       it "writes to the given column" do
-        expect(instance).to receive(:write_uploader).with(:monkey, [test_file_name])
         instance.images = [test_file_stub]
+        expect(instance).to receive(:write_uploader).with(:monkey, [test_file_name])
         instance.write_images_identifier
       end
 
