@@ -84,11 +84,11 @@ describe CarrierWave::Mount do
       end
 
       it "should inherit defined processors properly" do
-        expect(@uploader1.processors).to eq([[:rotate, [], nil]])
-        expect(@uploader2.processors).to eq([[:rotate, [], nil], [:shrink, [], nil]])
-        expect(@uploader1.versions[:thumb].processors).to eq([[:compress, [], nil]])
-        expect(@uploader2.versions[:thumb].processors).to eq([[:compress, [], nil]])
-        expect(@uploader2.versions[:secret].processors).to eq([[:encrypt, [], nil]])
+        expect(@uploader1.processors).to eq([[:rotate, [], nil, nil]])
+        expect(@uploader2.processors).to eq([[:rotate, [], nil, nil], [:shrink, [], nil, nil]])
+        expect(@uploader1.new.versions[:thumb].processors).to eq([[:compress, [], nil, nil]])
+        expect(@uploader2.new.versions[:thumb].processors).to eq([[:compress, [], nil, nil]])
+        expect(@uploader2.new.versions[:secret].processors).to eq([[:encrypt, [], nil, nil]])
       end
     end
 
@@ -135,19 +135,19 @@ describe CarrierWave::Mount do
         expect(@instance.image.current_path).to match(/^#{public_path('uploads/tmp')}/)
       end
 
-      it "should do nothing when nil is assigned" do
-        expect(@instance).not_to receive(:write_uploader)
+      it "should not change the attribute when nil is assigned" do
+        expect(@instance).to receive(:write_uploader).with(:image, nil)
         @instance.image = nil
       end
 
-      it "should do nothing when an empty string is assigned" do
-        expect(@instance).not_to receive(:write_uploader)
-        @instance.image = stub_file('test.jpg')
+      it "should not change the attribute when an empty string is assigned" do
+        expect(@instance).to receive(:write_uploader).with(:image, nil)
+        @instance.image = ''
       end
 
-      it "should fail silently if the image fails a white list integrity check" do
+      it "should fail silently if the image fails an allowlist integrity check" do
         @uploader.class_eval do
-          def extension_whitelist
+          def extension_allowlist
             %w(txt)
           end
         end
@@ -155,10 +155,10 @@ describe CarrierWave::Mount do
         expect(@instance.image).to be_blank
       end
 
-      it "should fail silently if the image fails a black list integrity check" do
+      it "should fail silently if the image fails a denylist integrity check" do
         @uploader.class_eval do
-          def extension_blacklist
-            %w(jpg)
+          def extension_allowlist
+            %w(png)
           end
         end
         @instance.image = stub_file('test.jpg')
@@ -175,6 +175,15 @@ describe CarrierWave::Mount do
         @instance.image = stub_file('test.jpg')
       end
 
+      it "should accept another uploader instance" do
+        @instance.image = stub_file('test.jpg')
+        @instance.image.store!
+        another = @class.new
+        another.image = @instance.image
+        expect(another.image).to be_an_instance_of(@uploader)
+        expect(another.image).to be_cached
+      end
+
     end
 
     describe '#image?' do
@@ -288,6 +297,13 @@ describe CarrierWave::Mount do
         @instance.image_cache = '1369894322-123-0123-1234/monkey.jpg'
         expect(@instance.image.current_path).to match(/test.jpg$/)
       end
+
+      it "should not clear a previously stored file when an empty string is assigned" do
+        @instance.image = stub_file('test.jpg')
+        @instance.image.store!
+        @instance.image_cache = ''
+        expect(@instance.image.current_path).to match(/test.jpg$/)
+      end
     end
 
     describe "#remote_image_url" do
@@ -348,12 +364,26 @@ describe CarrierWave::Mount do
         expect(@instance.image.current_path).to match(/test.jpg$/)
       end
 
-      it "writes over a previously assigned file" do
+      it "does not write over a previously assigned file" do
         @instance.image = stub_file("portrait.jpg")
         @instance.remote_image_url = "http://www.example.com/test.jpg"
 
+        expect(@instance.image.current_path).to match(/portrait.jpg$/)
+      end
+
+      it "does not clear a previously stored file when an empty string is assigned" do
+        @instance.remote_image_url = "http://www.example.com/test.jpg"
+        @instance.image.store!
+        @instance.remote_image_url = ""
         expect(@instance.image.current_path).to match(/test.jpg$/)
       end
+
+      it "clears the unsaved remote url when nil is assigned" do
+        @instance.remote_image_url = 'invalid'
+        @instance.remote_image_url = nil
+
+        expect(@instance.remote_image_url).to be_nil
+      end
     end
 
     describe '#store_image!' do
@@ -374,13 +404,21 @@ describe CarrierWave::Mount do
         expect(@instance.image.current_path).to eq(public_path('uploads/test.jpg'))
       end
 
-      it "should remove an uploaded file when remove_image? returns true" do
-        @instance.image = stub_file('test.jpg')
-        path = @instance.image.current_path
-        @instance.remove_image = true
-        @instance.store_image!
-        expect(@instance.image).to be_blank
-        expect(File.exist?(path)).to be_falsey
+      context "when adding a file which has the same filename with the existing one" do
+        before { FileUtils.cp(file_path('bork.txt'), tmp_path('test.jpg')) }
+        after { FileUtils.rm(tmp_path('test.jpg')) }
+
+        it "renames the latter file to avoid overwriting the old one" do
+          @instance.image = stub_file('test.jpg')
+          @instance.store_image!
+          old_uploader = @instance.image
+
+          @instance.image = File.open(tmp_path('test.jpg'))
+          @instance.write_image_identifier
+          @instance.store_image!
+          expect(@instance.image.identifier).to eq 'test(2).jpg'
+          expect(old_uploader.read).to eq 'this is stuff'
+        end
       end
     end
 
@@ -441,6 +479,11 @@ describe CarrierWave::Mount do
         expect(@instance.remove_image?).to be_falsey
       end
 
+      it "should be false when the value is 0" do
+        @instance.remove_image = 0
+        expect(@instance.remove_image?).to be_falsey
+      end
+
     end
 
     describe '#image_integrity_error' do
@@ -457,7 +500,7 @@ describe CarrierWave::Mount do
       describe "when an integrity check fails" do
         before do
           @uploader.class_eval do
-            def extension_whitelist
+            def extension_allowlist
               %w(txt)
             end
           end
@@ -568,8 +611,8 @@ describe CarrierWave::Mount do
 
     describe '#write_image_identifier' do
       it "should write to the column" do
-        expect(@instance).to receive(:write_uploader).with(:image, "test.jpg")
         @instance.image = stub_file('test.jpg')
+        expect(@instance).to receive(:write_uploader).with(:image, "test.jpg")
         @instance.write_image_identifier
       end
 
@@ -579,6 +622,7 @@ describe CarrierWave::Mount do
         @instance.remove_image = true
         expect(@instance).to receive(:write_uploader).with(:image, nil)
         @instance.write_image_identifier
+        expect(@instance.image).to be_blank
       end
     end
 
@@ -674,6 +718,10 @@ describe CarrierWave::Mount do
       it "should not apply any custom modifications to the uploader class" do
         expect(@uploader.new).not_to respond_to(:fish)
       end
+
+      it "should give the generated class a stable name" do
+        expect(@instance.image.class).to eq @class.const_get :CarrierWaveImageUploader
+      end
     end
   end
 
@@ -689,24 +737,24 @@ describe CarrierWave::Mount do
       @instance = @class.new
 
       @uploader.class_eval do
-        def extension_whitelist
+        def extension_allowlist
           %w(txt)
         end
       end
     end
 
     it "should raise an error if the image fails an integrity check when cached" do
-      expect(running {
+      expect {
         @instance.image = stub_file('test.jpg')
-      }).to raise_error(CarrierWave::IntegrityError)
+      }.to raise_error(CarrierWave::IntegrityError)
     end
 
     it "should raise an error if the image fails an integrity check when downloaded" do
       stub_request(:get, "http://www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
 
-      expect(running {
+      expect {
         @instance.remote_image_url = "http://www.example.com/test.jpg"
-      }).to raise_error(CarrierWave::IntegrityError)
+      }.to raise_error(CarrierWave::IntegrityError)
     end
   end
 
@@ -730,17 +778,17 @@ describe CarrierWave::Mount do
     end
 
     it "should raise an error if the image fails to be processed when cached" do
-      expect(running {
+      expect {
         @instance.image = stub_file('test.jpg')
-      }).to raise_error(CarrierWave::ProcessingError)
+      }.to raise_error(CarrierWave::ProcessingError)
     end
 
     it "should raise an error if the image fails to be processed when downloaded" do
       stub_request(:get, "http://www.example.com/test.jpg").to_return(body: File.read(file_path("test.jpg")))
 
-      expect(running {
+      expect {
         @instance.remote_image_url = "http://www.example.com/test.jpg"
-      }).to raise_error(CarrierWave::ProcessingError)
+      }.to raise_error(CarrierWave::ProcessingError)
     end
 
   end
@@ -759,14 +807,14 @@ describe CarrierWave::Mount do
 
     it "should raise an error if the image fails to be processed" do
       @uploader.class_eval do
-        def download! uri, headers = {}
+        def download!(uri, headers = {})
           raise CarrierWave::DownloadError
         end
       end
 
-      expect(running {
+      expect {
         @instance.remote_image_url = "http://www.example.com/test.jpg"
-      }).to raise_error(CarrierWave::DownloadError)
+      }.to raise_error(CarrierWave::DownloadError)
     end
 
   end
@@ -793,8 +841,8 @@ describe CarrierWave::Mount do
 
     describe '#write_image_identifier' do
       it "should write to the given column" do
-        expect(@instance).to receive(:write_uploader).with(:monkey, "test.jpg")
         @instance.image = stub_file('test.jpg')
+        expect(@instance).to receive(:write_uploader).with(:monkey, "test.jpg")
         @instance.write_image_identifier
       end
 
