@@ -23,14 +23,20 @@ describe CarrierWave::MiniMagick do
       expect(instance.file.content_type).to eq('image/png')
     end
 
-    it "converts all pages when no page number is specified" do
-      expect_any_instance_of(::MiniMagick::Image).to receive(:format).with('png', nil).once
-      instance.convert('png')
-    end
+    it "respects the page parameter" do
+      # create a multi-layer image
+      tiff = Tempfile.new(["file", ".tiff"])
+      MiniMagick::Tool::Convert.new do |convert|
+        convert.merge! [landscape_file_path, landscape_file_path, landscape_file_path]
+        convert << tiff.path
+      end
 
-    it "converts specific page" do
-      expect_any_instance_of(::MiniMagick::Image).to receive(:format).with('png', 1).once
-      instance.convert('png', 1)
+      allow(instance).to receive(:file).and_return(CarrierWave::SanitizedFile.new(tiff.path))
+
+      instance.convert('png', 0)
+      expect(instance.file.extension).to eq('png')
+      expect(instance).to be_format('png')
+      expect(instance.file.size).not_to eq(0)
     end
   end
 
@@ -222,16 +228,16 @@ describe CarrierWave::MiniMagick do
       before { File.open(instance.current_path, 'w') { |f| f.puts "bogus" } }
 
       it "fails to process a non image file" do
-        expect { instance.resize_to_limit(200, 200) }.to raise_exception(CarrierWave::ProcessingError, /^Failed to manipulate with MiniMagick, maybe it is not an image\?/)
+        expect { instance.resize_to_limit(200, 200) }.to raise_exception(CarrierWave::ProcessingError, /^Failed to manipulate, maybe it is not an image\?/)
       end
 
       it "uses I18n" do
         change_locale_and_store_translations(:nl, :errors => {
           :messages => {
-            :mini_magick_processing_error => "Kon bestand niet met MiniMagick bewerken, misschien is het geen beeld bestand?"
+            :processing_error => "Kon bestand niet bewerken, misschien is het geen beeld bestand?"
           }
         }) do
-          expect {instance.resize_to_limit(200, 200)}.to raise_exception(CarrierWave::ProcessingError, /^Kon bestand niet met MiniMagick bewerken, misschien is het geen beeld bestand\?/)
+          expect {instance.resize_to_limit(200, 200)}.to raise_exception(CarrierWave::ProcessingError, /^Kon bestand niet bewerken, misschien is het geen beeld bestand\?/)
         end
       end
 
@@ -243,11 +249,75 @@ describe CarrierWave::MiniMagick do
 
       context ":en locale is not available and enforce_available_locales is true" do
         it "doesn't suppress errors" do
-          change_and_enforece_available_locales(:nl, [:nl, :foo]) do
+          change_and_enforce_available_locales(:nl, [:nl, :foo]) do
             expect { instance.resize_to_limit(200, 200) }.to raise_exception(CarrierWave::ProcessingError)
           end
         end
       end
     end
+
+    context "of failing to find ImageMagick/GraphicsMagick" do
+      before do
+        MiniMagick.remove_instance_variable(:@processor) if MiniMagick.instance_variable_defined?(:@processor)
+        MiniMagick.remove_instance_variable(:@cli) if MiniMagick.instance_variable_defined?(:@cli)
+        allow(MiniMagick::Utilities).to receive(:which).and_return(nil)
+      end
+
+      it "raises MiniMagick::Error" do
+        expect { instance.resize_to_limit(200, 200) }.to raise_exception(MiniMagick::Error)
+      end
+    end
+
+    context "of being configured to use ImageMagick but failing to execute" do
+      before do
+        allow(MiniMagick).to receive(:processor).and_return(:magick)
+        allow_any_instance_of(MiniMagick::Shell).to receive(:execute_open3).and_raise(Errno::ENOENT)
+      end
+
+      it "raises MiniMagick::Error" do
+        expect { instance.resize_to_limit(200, 200) }.to raise_exception(MiniMagick::Error)
+      end
+    end
+  end
+
+  describe "#manipulate!" do
+    it "performs manipulation using the given block" do
+      instance.manipulate! do |image|
+        image.format('png')
+      end
+      expect(instance).to be_format('png')
+    end
+
+    context "on failing to find ImageMagick/GraphicsMagick" do
+      before do
+        MiniMagick.remove_instance_variable(:@processor) if MiniMagick.instance_variable_defined?(:@processor)
+        MiniMagick.remove_instance_variable(:@cli) if MiniMagick.instance_variable_defined?(:@cli)
+        allow(MiniMagick::Utilities).to receive(:which).and_return(nil)
+      end
+
+      it "raises MiniMagick::Invalid" do
+        expect do
+          instance.manipulate! do |image|
+            image.format('png')
+          end
+        end.to raise_exception(MiniMagick::Invalid)
+      end
+    end
+
+    context "on being configured to use ImageMagick but failing to execute" do
+      before do
+        allow(MiniMagick).to receive(:processor).and_return(:magick)
+        allow_any_instance_of(MiniMagick::Shell).to receive(:execute_open3).and_raise(Errno::ENOENT)
+      end
+      after { MiniMagick.remove_instance_variable(:@processor) }
+
+      it "raises MiniMagick::Invalid" do
+        expect do
+          instance.manipulate! do |image|
+            image.format('png')
+          end
+        end.to raise_exception(MiniMagick::Invalid)
+      end
+    end
   end
 end
