@@ -1,12 +1,5 @@
 require 'spec_helper'
 
-begin
-  # Use mime/types/columnar if available, for reduced memory usage
-  require 'mime/types/columnar'
-rescue LoadError
-  require 'mime/types'
-end
-
 describe CarrierWave::SanitizedFile do
   before do
     FileUtils.cp(file_path('test.jpg'), file_path('llama.jpg'))
@@ -158,6 +151,11 @@ describe CarrierWave::SanitizedFile do
       expect(sanitized_file).to receive(:original_filename).at_least(:once).and_return("ТестоВый Ёжик.jpg")
       expect(sanitized_file.filename).to eq("ТестоВый_Ёжик.jpg")
     end
+
+    it "should remove invalid byte sequences from the filename" do
+      expect(sanitized_file).to receive(:original_filename).at_least(:once).and_return("test\xDD.jpg")
+      expect(sanitized_file.filename).to eq("test_.jpg")
+    end
   end
 
   describe "#filename with an overridden sanitize_regexp" do
@@ -193,7 +191,8 @@ describe CarrierWave::SanitizedFile do
 
     it "handles Mime::Type object" do
       file = File.open(file_path('sponsored.doc'))
-      allow(file).to receive(:content_type).and_return(MIME::Type.new("application/msword"))
+      allow(file).to receive(:content_type) { 'application/msword' }
+
       sanitized_file = CarrierWave::SanitizedFile.new(file)
       allow(sanitized_file).to receive(:file).and_return(file)
 
@@ -201,11 +200,139 @@ describe CarrierWave::SanitizedFile do
       expect(sanitized_file.content_type).to eq("application/msword")
     end
 
-    it "reads content type from path if missing" do
+    it "handles Mime::Type of docx" do
+      file = File.open(file_path('resume.docx'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+      allow(sanitized_file).to receive(:file).and_return(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+      expect(sanitized_file.content_type).to eq("application/vnd.openxmlformats-officedocument.wordprocessingml.document")
+    end
+
+    it "handles Mime::Type of pptx" do
+      file = File.open(file_path('slidedeck.pptx'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+      allow(sanitized_file).to receive(:file).and_return(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+      expect(sanitized_file.content_type).to eq("application/vnd.openxmlformats-officedocument.presentationml.presentation")
+    end
+
+    it "does not raise an error if missing" do
       sanitized_file = CarrierWave::SanitizedFile.new("llama.jpg")
 
+      expect(sanitized_file.content_type).to eq("application/octet-stream")
+    end
+
+    it "reads content type of a CSV linked to a file" do
+      file = File.open(file_path('addresses.csv'))
+      csv_file = CSV.new(file)
+      sanitized_file = CarrierWave::SanitizedFile.new(csv_file)
+
+      expect(sanitized_file.content_type).to eq("text/csv")
+    end
+
+    it "does not allow spoofing of the mime type passed by Hash" do
+      file = File.open(file_path("landscape.jpg"))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(tempfile: file, content_type: 'image/png')
+
       expect(sanitized_file.content_type).to eq("image/jpeg")
     end
+
+    it "does not allow spoofing of the mime type by setting file's content type" do
+      file = File.open(file_path("landscape.jpg"))
+      allow(file).to receive(:content_type) { 'image/png' }
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect(sanitized_file.content_type).to eq("image/jpeg")
+    end
+
+    it "does not allow spoofing of the mime type by file extension" do
+      file = File.open(file_path("zip.png"))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+      expect { sanitized_file.content_type }.not_to raise_error
+
+      expect(sanitized_file.content_type).to eq("application/zip")
+    end
+
+    it "does not allow spoofing of the mime type if the mime type is not detectable" do
+      file = File.open(file_path('spoof.png'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+
+      expect(sanitized_file.content_type).to_not eq 'image/png'
+      expect(sanitized_file.content_type).to eq 'application/octet-stream'
+    end
+
+    it "returns valid content type on text file" do
+      file = File.open(file_path('bork.txt'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+
+      expect(sanitized_file.content_type).to eq 'text/plain'
+    end
+
+    it "returns valid content type on JSON file (which is a text file in disguise)" do
+      file = File.open(file_path('bork.json'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+
+      expect(sanitized_file.content_type).to eq 'application/json'
+    end
+
+    it "returns binary content type with unknown extension" do
+      file = File.open(file_path('bork.ABCDE'))
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+
+      expect(sanitized_file.content_type).to eq "application/octet-stream"
+    end
+
+    it "does not raise an error if the path is not present" do
+      sanitized_file = CarrierWave::SanitizedFile.new(nil)
+
+      expect { sanitized_file.content_type }.not_to raise_error
+    end
+
+    it "uses the first one when multiple mime types are given using a semicolon" do
+      file = File.open(file_path("bork.txt"))
+      allow(file).to receive(:content_type) { 'image/png; text/html' }
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect(sanitized_file.content_type).to eq("image/png")
+    end
+
+    it "uses the first one when multiple mime types are given using a comma" do
+      file = File.open(file_path("bork.txt"))
+      allow(file).to receive(:content_type) { 'image/png, text/html' }
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect(sanitized_file.content_type).to eq("image/png")
+    end
+
+    it "drops content type parameters" do
+      file = File.open(file_path("bork.txt"))
+      allow(file).to receive(:content_type) { 'text/html; charset=utf-8' }
+
+      sanitized_file = CarrierWave::SanitizedFile.new(file)
+
+      expect(sanitized_file.content_type).to eq("text/html")
+    end
   end
 
   describe "#content_type=" do
@@ -252,6 +379,17 @@ describe CarrierWave::SanitizedFile do
       it "should return the contents of the file" do
         expect(sanitized_file.read).to eq("this is stuff")
       end
+
+      it "accepts the optional arguments length and outbuf" do
+        buf = ""
+        sanitized_file.read(4, buf)
+        expect(buf).to eq("this")
+      end
+
+      it "always reads from the file if arguments are given" do
+        sanitized_file.read
+        expect(sanitized_file.read(4)).to eq("this")
+      end
     end
 
     describe "#size" do
@@ -296,25 +434,41 @@ describe CarrierWave::SanitizedFile do
       end
 
       it "should set the right permissions" do
-        sanitized_file.move_to(file_path("gurr.png"), 0755)
+        sanitized_file.move_to(file_path("gurr.png"), 0o755)
 
-        expect(sanitized_file).to have_permissions(0755)
+        expect(sanitized_file).to have_permissions(0o755)
       end
 
       it "should set the right directory permissions" do
-        sanitized_file.move_to(file_path("new_dir","gurr.png"), nil, 0775)
+        sanitized_file.move_to(file_path("new_dir", "gurr.png"), nil, 0o775)
 
-        expect(sanitized_file).to have_directory_permissions(0775)
+        expect(sanitized_file).to have_directory_permissions(0o775)
       end
 
       it "should return itself" do
         expect(sanitized_file.move_to(file_path("gurr.png"))).to eq(sanitized_file)
       end
 
-      it "should convert the file's content type" do
-        sanitized_file.move_to(file_path("new_dir","gurr.png"))
+      it "should preserve the file's content type" do
+        sanitized_file.content_type = 'application/x-something'
+        sanitized_file.move_to(file_path("new_dir", "gurr.png"))
 
-        expect(sanitized_file.content_type).to eq("image/jpeg")
+        expect(sanitized_file.content_type).to eq("application/x-something")
+      end
+
+      it "should detect content type using Magic when content_type is reset" do
+        sanitized_file.content_type = nil
+        sanitized_file.move_to(file_path("new_dir", "gurr.png"))
+
+        expect(sanitized_file).to receive(:identified_content_type)
+        sanitized_file.content_type
+      end
+
+      it "should not try to identify content type if unnecessary" do
+        expect(sanitized_file).not_to receive(:identified_content_type)
+        sanitized_file.move_to(file_path("new_dir", "gurr.png"))
+
+        expect(sanitized_file.instance_variable_get(:@content_type)).to be nil
       end
 
       context 'target path only differs by case' do
@@ -326,13 +480,13 @@ describe CarrierWave::SanitizedFile do
           expect(upcased_sanitized_file).not_to be_empty
         end
 
-	after(:all) do
-	  FileUtils.rm_f(file_path("upcase.JPG"))
-	  FileUtils.rm_f(file_path("upcase.jpg"))
+        after do
+          FileUtils.rm_f(file_path("upcase.JPG"))
+          FileUtils.rm_f(file_path("upcase.jpg"))
         end
 
         it "should not raise an error when moved" do
-          expect(running { upcased_sanitized_file.move_to(upcased_sanitized_file.path.downcase) }).not_to raise_error
+          expect { upcased_sanitized_file.move_to(upcased_sanitized_file.path.downcase) }.not_to raise_error
         end
       end
     end
@@ -351,11 +505,11 @@ describe CarrierWave::SanitizedFile do
       end
 
       it "should not have changed its path when copied" do
-        expect(running { sanitized_file.copy_to(file_path("gurr.png")) }).not_to change(sanitized_file, :path)
+        expect { sanitized_file.copy_to(file_path("gurr.png")) }.not_to change(sanitized_file, :path)
       end
 
       it "should not have changed its filename when copied" do
-        expect(running { sanitized_file.copy_to(file_path("gurr.png")) }).not_to change(sanitized_file, :filename)
+        expect { sanitized_file.copy_to(file_path("gurr.png")) }.not_to change(sanitized_file, :filename)
       end
 
       it "should return an object of the same class when copied" do
@@ -389,15 +543,15 @@ describe CarrierWave::SanitizedFile do
       end
 
       it "should set the right permissions" do
-        new_file = sanitized_file.copy_to(file_path("gurr.png"), 0755)
+        new_file = sanitized_file.copy_to(file_path("gurr.png"), 0o755)
 
-        expect(new_file).to have_permissions(0755)
+        expect(new_file).to have_permissions(0o755)
       end
 
       it "should set the right directory permissions" do
-        new_file = sanitized_file.copy_to(file_path("new_dir", "gurr.png"), nil, 0755)
+        new_file = sanitized_file.copy_to(file_path("new_dir", "gurr.png"), nil, 0o755)
 
-        expect(new_file).to have_directory_permissions(0755)
+        expect(new_file).to have_directory_permissions(0o755)
       end
 
       it "should preserve the file's content type" do
@@ -405,13 +559,28 @@ describe CarrierWave::SanitizedFile do
 
         expect(new_file.content_type).to eq(sanitized_file.content_type)
       end
+
+      it "should not try to identify content type if unnecessary" do
+        expect(sanitized_file).not_to receive(:identified_content_type)
+        new_file = sanitized_file.copy_to(file_path("new_dir", "gurr.png"))
+
+        expect(new_file.instance_variable_get(:@content_type)).to be nil
+      end
+    end
+
+    describe "#with_io" do
+      it "should yield an IO object" do
+        sanitized_file.send(:with_io) do |io|
+          expect(io).to respond_to(:read)
+        end
+      end
     end
   end
 
   shared_examples_for "all valid sanitized files that are stored on disk" do
     describe "#move_to" do
       it "should not raise an error when moved to its own location" do
-        expect(running { sanitized_file.move_to(sanitized_file.path) }).not_to raise_error
+        expect { sanitized_file.move_to(sanitized_file.path) }.not_to raise_error
       end
 
       it "should remove the original file" do
@@ -424,10 +593,10 @@ describe CarrierWave::SanitizedFile do
 
     describe '#copy_to' do
       it "should return a new instance when copied to its own location" do
-        expect(running {
+        expect {
           new_file = sanitized_file.copy_to(sanitized_file.path)
           expect(new_file).to be_an_instance_of(sanitized_file.class)
-        }).not_to raise_error
+        }.not_to raise_error
       end
 
       it "should not remove the original file" do
@@ -463,19 +632,33 @@ describe CarrierWave::SanitizedFile do
         expect(sanitized_file.to_file.path).to eq(sanitized_file.path)
       end
 
-      it "should have the same contents as the SantizedFile" do
+      it "should have the same contents as the SanitizedFile" do
         expect(sanitized_file.to_file.read).to eq(sanitized_file.read)
       end
     end
   end
 
   shared_examples_for "all valid sanitized files that are read from an IO object" do
-
     describe "#read" do
       it "should have an open IO object" do
         expect(sanitized_file.instance_variable_get(:@file).closed?).to be_falsey
       end
+    end
 
+    describe "#with_io" do
+      it "should try to rewind the IO object on finish" do
+        sanitized_file.send(:with_io) do |io|
+          io.read
+        end
+        expect(sanitized_file.file.pos).to eq 0
+      end
+    end
+  end
+
+  shared_examples_for "all valid sanitized files that are read from a non-StringIO object" do
+    it_should_behave_like "all valid sanitized files that are read from an IO object"
+
+    describe "#read" do
       it "should close the IO object after reading" do
         sanitized_file.read
 
@@ -484,6 +667,18 @@ describe CarrierWave::SanitizedFile do
     end
   end
 
+  shared_examples_for "all valid sanitized files that are read from a StringIO object" do
+    it_should_behave_like "all valid sanitized files that are read from an IO object"
+
+    describe "#read" do
+      it "should not close StringIO inherited object after reading" do
+        sanitized_file.read
+
+        expect(sanitized_file.instance_variable_get(:@file).closed?).to be_falsey
+      end
+    end
+  end
+
   describe "with a valid Hash" do
     let(:hash) {
       {
@@ -498,7 +693,7 @@ describe CarrierWave::SanitizedFile do
 
     it_should_behave_like "all valid sanitized files that are stored on disk"
 
-    it_should_behave_like "all valid sanitized files that are read from an IO object"
+    it_should_behave_like "all valid sanitized files that are read from a non-StringIO object"
 
     describe "#path" do
       it "should return the path of the tempfile" do
@@ -522,7 +717,7 @@ describe CarrierWave::SanitizedFile do
 
     it_should_behave_like "all valid sanitized files that are stored on disk"
 
-    it_should_behave_like "all valid sanitized files that are read from an IO object"
+    it_should_behave_like "all valid sanitized files that are read from a non-StringIO object"
 
     describe "#is_path?" do
       it "should be false" do
@@ -543,7 +738,7 @@ describe CarrierWave::SanitizedFile do
 
     it_should_behave_like "all valid sanitized files"
 
-    it_should_behave_like "all valid sanitized files that are read from an IO object"
+    it_should_behave_like "all valid sanitized files that are read from a StringIO object"
 
     describe "#exists?" do
       it "should be false" do
@@ -565,7 +760,7 @@ describe CarrierWave::SanitizedFile do
 
     describe "#delete" do
       it "should not raise an error" do
-        expect(running { sanitized_file.delete }).not_to raise_error
+        expect { sanitized_file.delete }.not_to raise_error
       end
     end
 
@@ -589,7 +784,7 @@ describe CarrierWave::SanitizedFile do
 
     it_should_behave_like "all valid sanitized files that are stored on disk"
 
-    it_should_behave_like "all valid sanitized files that are read from an IO object"
+    it_should_behave_like "all valid sanitized files that are read from a non-StringIO object"
 
     describe "#is_path?" do
       it "should be false" do
@@ -618,7 +813,7 @@ describe CarrierWave::SanitizedFile do
 
     it_should_behave_like "all valid sanitized files that are stored on disk"
 
-    it_should_behave_like "all valid sanitized files that are read from an IO object"
+    it_should_behave_like "all valid sanitized files that are read from a non-StringIO object"
 
     describe "#is_path?" do
       it "should be false" do
@@ -690,6 +885,32 @@ describe CarrierWave::SanitizedFile do
 
   end
 
+  describe "with an uploader instance" do
+    let(:uploader_class) { Class.new(CarrierWave::Uploader::Base) }
+    let(:uploader) do
+      uploader_class.new.tap do |instance|
+        instance.store!(File.open(file_path('llama.jpg')))
+      end
+    end
+
+    let(:sanitized_file) { CarrierWave::SanitizedFile.new(uploader) }
+
+    it_should_behave_like "all valid sanitized files"
+
+    describe "#is_path?" do
+      it "should be false" do
+        expect(sanitized_file.is_path?).to be_falsey
+      end
+    end
+
+    describe "#path" do
+      it "should return the path of the stored file" do
+        expect(sanitized_file.path).not_to be_nil
+        expect(sanitized_file.path).to eq(public_path("uploads/llama.jpg"))
+      end
+    end
+  end
+
   describe "that is empty" do
     let(:empty) { CarrierWave::SanitizedFile.new(nil) }
 
@@ -749,7 +970,7 @@ describe CarrierWave::SanitizedFile do
 
     describe "#delete" do
       it "should not raise an error" do
-        expect(running { empty.delete }).not_to raise_error
+        expect { empty.delete }.not_to raise_error
       end
     end
 
@@ -819,7 +1040,7 @@ describe CarrierWave::SanitizedFile do
 
     describe "#delete" do
       it "should not raise an error" do
-        expect(running { empty.delete }).not_to raise_error
+        expect { empty.delete }.not_to raise_error
       end
     end
 
