@@ -1,23 +1,29 @@
 require 'rubygems'
 require 'bundler/setup'
 
-require 'pry'
+if RUBY_ENGINE == 'jruby'
+  # Workaround for JRuby CI failure https://github.com/jruby/jruby/issues/6547#issuecomment-774104996
+  require 'i18n/backend'
+  require 'i18n/backend/simple'
+end
+
+require 'pry' unless ENV['GITHUB_ACTIONS']
 require 'tempfile'
 require 'time'
 require 'logger'
+require 'csv'
 
 require 'carrierwave'
 require 'timecop'
 require 'open-uri'
 require "webmock/rspec"
 require 'mini_magick'
+require "vips"
+require 'active_support/core_ext'
+require 'rspec/retry'
 
 I18n.enforce_available_locales = false
 
-CARRIERWAVE_DIRECTORY = "carrierwave#{Time.now.to_i}" unless defined?(CARRIERWAVE_DIRECTORY)
-
-alias :running :lambda
-
 def file_path( *paths )
   File.expand_path(File.join(File.dirname(__FILE__), 'fixtures', *paths))
 end
@@ -50,7 +56,7 @@ module CarrierWave
         tempfile = Tempfile.new(filename)
         FileUtils.copy_file(file_path(filename), tempfile.path)
         allow(tempfile).to receive_messages(:original_filename => fake_name || filename,
-                      :content_type => mime_type)
+                                            :content_type => mime_type)
         tempfile
       end
 
@@ -60,13 +66,15 @@ module CarrierWave
         file = IO.read( file_path( filename ) ) if filename
         stringio = StringIO.new(file)
         allow(stringio).to receive_messages(:local_path => "",
-                      :original_filename => filename || fake_name,
-                      :content_type => mime_type)
+                                            :original_filename => filename || fake_name,
+                                            :content_type => mime_type)
         stringio
       end
 
       def stub_file(filename, mime_type=nil, fake_name=nil)
-        File.open(file_path(filename))
+        f = File.open(file_path(filename))
+        allow(f).to receive(:content_type) { mime_type } if mime_type
+        f
       end
     end
 
@@ -86,7 +94,7 @@ module CarrierWave
         end
       end
 
-      def change_and_enforece_available_locales(locale, available_locales, &block)
+      def change_and_enforce_available_locales(locale, available_locales, &block)
         current_available_locales = I18n.available_locales
         current_enforce_available_locales_value = I18n.enforce_available_locales
         current_locale = I18n.locale
@@ -111,10 +119,31 @@ module CarrierWave
     end
 
     module SsrfProtectionAwareWebMock
+      class Matcher
+        def initialize(uri)
+          @uri = uri
+        end
+
+        def call(target_uri)
+          Resolv.getaddresses(@uri.hostname).any? do |address|
+            candidate = @uri.dup
+            candidate.hostname = address
+            target_uri == WebMock::Util::URI.normalize_uri(candidate)
+          end
+        end
+
+        def inspect
+          "#<#{self.class.name}: #{@uri}>"
+        end
+      end
+
       def stub_request(method, uri)
         uri = URI.parse(uri) if uri.is_a?(String)
-        uri.hostname = Resolv.getaddress(uri.hostname) if uri.is_a?(URI)
-        super
+        if uri.is_a?(URI)
+          super method, Matcher.new(uri)
+        else
+          super
+        end
       end
     end
   end
@@ -127,6 +156,15 @@ RSpec.configure do |config|
   config.include CarrierWave::Test::I18nHelpers
   config.include CarrierWave::Test::ManipulationHelpers
   config.prepend CarrierWave::Test::SsrfProtectionAwareWebMock
+  config.verbose_retry = true
+  config.display_try_failure_messages = true
+  config.raise_errors_for_deprecations!
+  config.around :each, :with_retry do |example|
+    example.run_with_retry retry: 2
+  end
+  config.retry_callback = proc do |example|
+    sleep 1
+  end
   if RUBY_ENGINE == 'jruby'
     config.filter_run_excluding :rmagick => true
   end
