@@ -20,7 +20,7 @@ describe CarrierWave::Storage::File do
     context "when the directory is not empty" do
       let(:cache_id_dir) { File.dirname(cache_path) }
       let(:cache_path) { File.expand_path(File.join(uploader.cache_dir, uploader.cache_name), uploader.root) }
-      let(:existing_file) { File.join(cache_id_dir, "exsting_file.txt") }
+      let(:existing_file) { File.join(cache_id_dir, "existing_file.txt") }
 
       before do
         uploader.cache!(file)
@@ -63,7 +63,7 @@ describe CarrierWave::Storage::File do
 
     before do
       [five_days_ago, three_days_ago, yesterday, (today - 1.minute)].each do |created_date|
-        Timecop.freeze (created_date) do
+        Timecop.freeze(created_date) do
           FileUtils.mkdir_p File.expand_path(CarrierWave.generate_cache_id, cache_dir)
         end
       end
@@ -94,5 +94,34 @@ describe CarrierWave::Storage::File do
 
       expect(Dir.glob("#{cache_dir}/*").size).to eq(2)
     end
+
+    it "cleans a directory named using old format of cache id" do
+      FileUtils.mkdir_p File.expand_path("#{yesterday.utc.to_i}-100-1234", cache_dir)
+      Timecop.freeze(today) { uploader_class.clean_cached_files!(0) }
+
+      expect(Dir.glob("#{cache_dir}/*").size).to eq(0)
+    end
+
+    it "clears all files older than now in the new cache directory" do
+      uploader_class.root = File.expand_path("new", CarrierWave.root)
+      new_cache_dir = File.expand_path(uploader_class.cache_dir, uploader.root)
+      Timecop.freeze(today - 1.minute) { FileUtils.mkdir_p File.expand_path(CarrierWave.generate_cache_id, new_cache_dir) }
+
+      Timecop.freeze(today) { uploader_class.clean_cached_files!(0) }
+      expect(Dir.glob("#{new_cache_dir}/*").size).to eq(0)
+    end
+
+    context "when a file which does not conform to the cache_id format exists" do
+      before do
+        FileUtils.touch File.expand_path("invalid", cache_dir)
+      end
+
+      it "should just ignore that" do
+        Timecop.freeze(today) { uploader_class.clean_cached_files!(0) }
+
+        expect(Dir.glob("#{cache_dir}/*").size).to eq(1)
+        expect(File).to exist("#{cache_dir}/invalid")
+      end
+    end
   end
 end
