@@ -4,11 +4,12 @@ describe CarrierWave::Uploader do
   describe "callback isolation" do
     let(:default_before_callbacks) do
       [
-        :check_extension_whitelist!,
-        :check_extension_blacklist!,
-        :check_content_type_whitelist!,
-        :check_content_type_blacklist!,
+        :check_extension_allowlist!,
+        :check_extension_denylist!,
+        :check_content_type_allowlist!,
+        :check_content_type_denylist!,
         :check_size!,
+        :check_dimensions!,
         :process!
       ]
     end
@@ -20,7 +21,6 @@ describe CarrierWave::Uploader do
 
     it { expect(uploader_class_1._before_callbacks[:cache]).to eq(default_before_callbacks) }
 
-
     it { expect(uploader_class_2._before_callbacks[:cache]).to eq(default_before_callbacks + [:before_cache_callback]) }
 
     it "doesn't inherit the uploader 2 callback" do
