@@ -9,7 +9,7 @@ describe CarrierWave::Uploader do
   after { FileUtils.rm_rf(public_path) }
 
   describe '#cache!' do
-    subject { lambda { uploader.cache!(test_file) } }
+    subject { uploader.cache!(test_file) }
 
     before { allow(CarrierWave).to receive(:generate_cache_id).and_return(cache_id) }
 
@@ -20,7 +20,7 @@ describe CarrierWave::Uploader do
         let(:range) { nil }
 
         it "doesn't raise an integrity error" do
-          is_expected.not_to raise_error
+          expect { subject }.not_to raise_error
         end
       end
 
@@ -28,7 +28,7 @@ describe CarrierWave::Uploader do
         let(:range) { 2097152..4194304 }
 
         it "raises an integrity error" do
-          is_expected.to raise_error(CarrierWave::IntegrityError, 'File size should be greater than 2 MB')
+          expect { subject }.to raise_error(CarrierWave::IntegrityError, 'File size should be greater than 2 MB')
         end
       end
 
@@ -36,7 +36,7 @@ describe CarrierWave::Uploader do
         let(:range) { 0..10 }
 
         it "raises an integrity error" do
-          is_expected.to raise_error(CarrierWave::IntegrityError, 'File size should be less than 10 Bytes')
+          expect { subject }.to raise_error(CarrierWave::IntegrityError, 'File size should be less than 10 Bytes')
         end
       end
 
@@ -44,7 +44,7 @@ describe CarrierWave::Uploader do
         let(:range) { 0..100 }
 
         it "doesn't raise an integrity error" do
-          is_expected.not_to raise_error
+          expect { subject }.not_to raise_error
         end
       end
     end
