@@ -8,14 +8,28 @@ describe CarrierWave::Uploader do
   end
 
   after do
+    @uploader_class.constants
+      .select { |const| const.to_s =~ /Uploader$/ }
+      .each { |const| @uploader_class.send(:remove_const, const)}
     FileUtils.rm_rf(public_path)
   end
 
   describe '.version' do
-    it "should add it to .versions" do
+    it "should add the builder to .versions" do
       @uploader_class.version :thumb
-      expect(@uploader_class.versions[:thumb]).to be_a(Class)
-      expect(@uploader_class.versions[:thumb].ancestors).to include(@uploader_class)
+      expect(@uploader_class.versions[:thumb]).to be_a(CarrierWave::Uploader::Versions::Builder)
+    end
+
+    it "should raise an error when a user tries to use a Builder for configuration" do
+      @uploader_class.version :thumb
+      expect { @uploader_class.versions[:thumb].storage = :file }.to raise_error NoMethodError, /{ self.storage= :file }/
+      expect { @uploader_class.versions[:thumb].process convert: :png }.to raise_error NoMethodError, /{ self.process {:convert=>:png} }/
+    end
+
+    it "should add an version instance to #versions" do
+      @uploader_class.version :thumb
+      expect(@uploader.versions[:thumb]).to be_a(CarrierWave::Uploader::Base)
+      expect(@uploader.versions[:thumb].class.ancestors).to include(@uploader_class)
     end
 
     it "should only assign versions to parent" do
@@ -56,6 +70,11 @@ describe CarrierWave::Uploader do
       expect(@uploader.thumb.class.version_names).to eq([:thumb])
     end
 
+    it "should set the class name" do
+      @uploader_class.version :thumb
+      expect(@uploader.thumb.class).to eq @uploader_class.const_get :ThumbVersionUploader
+    end
+
     it "should remember mount options" do
       model = double('a model')
       @uploader_class.version :thumb
@@ -76,21 +95,27 @@ describe CarrierWave::Uploader do
     end
 
     it "should not initially have a value for enable processing" do
-      thumb = (@uploader_class.version :thumb)
-      expect(thumb.instance_variable_get('@enable_processing')).to be_nil
+      @uploader_class.version :thumb
+      expect(@uploader.thumb.class.instance_variable_get('@enable_processing')).to be_nil
     end
 
     it "should return the enable processing value of the parent" do
+      @uploader_class.version :thumb
       @uploader_class.enable_processing = false
-      thumb = (@uploader_class.version :thumb)
-      expect(thumb.enable_processing).to be_falsey
+      expect(@uploader.thumb.class.enable_processing).to be_falsey
     end
 
     it "should return its own value for enable processing if set" do
       @uploader_class.enable_processing = false
-      thumb = @uploader_class.version :thumb
-      thumb.enable_processing = true
-      expect(thumb.enable_processing).to be_truthy
+      @uploader_class.version(:thumb) { self.enable_processing = true }
+      expect(@uploader.thumb.enable_processing).to be_truthy
+    end
+
+    it "should use the enable processing value of the parent after reading its own value" do
+      @uploader_class.version :thumb
+      @uploader.cache!(File.open(file_path('test.jpg')))
+      @uploader_class.enable_processing = false
+      expect(@uploader.thumb.class.enable_processing).to be_falsey
     end
 
     it "should reopen the same class when called multiple times" do
@@ -104,8 +129,8 @@ describe CarrierWave::Uploader do
           "llama"
         end
       end
-      expect(@uploader_class.version(:thumb).monkey).to eq("monkey")
-      expect(@uploader_class.version(:thumb).llama).to eq("llama")
+      expect(@uploader.thumb.class.monkey).to eq("monkey")
+      expect(@uploader.thumb.class.llama).to eq("llama")
     end
 
     it "should reopen the same instance when called multiple times" do
@@ -125,7 +150,7 @@ describe CarrierWave::Uploader do
 
     it "should accept option :from_version" do
       @uploader_class.version :small_thumb, :from_version => :thumb
-      expect(@uploader_class.version(:small_thumb).version_options[:from_version]).to eq(:thumb)
+      expect(@uploader.small_thumb.class.version_options[:from_version]).to eq(:thumb)
     end
 
     describe 'with nested versions' do
@@ -187,11 +212,17 @@ describe CarrierWave::Uploader do
     describe 'with inheritance' do
 
       before do
+        @uploader_class.class_eval do
+          def store_dir
+            public_path('monkey')
+          end
+        end
         @uploader_class.version :thumb do
           def store_dir
             public_path('monkey/apache')
           end
         end
+        @uploader_class.version :preview
 
         @child_uploader_class = Class.new(@uploader_class)
         @child_uploader = @child_uploader_class.new
@@ -216,6 +247,27 @@ describe CarrierWave::Uploader do
 
         expect(@uploader.thumb.store_dir).to eq(public_path('monkey/apache'))
       end
+
+      it "should respect store_dir in the subclass" do
+        @child_uploader_class.class_eval do
+          def store_dir
+            public_path('gorilla')
+          end
+        end
+
+        expect(@uploader.preview.store_dir).to eq(public_path('monkey'))
+        expect(@child_uploader.preview.store_dir).to eq(public_path('gorilla'))
+      end
+
+      it "should respect default_url in the subclass" do
+        @child_uploader_class.class_eval do
+          def default_url
+            "/#{version_name}.png"
+          end
+        end
+
+        expect(@child_uploader.preview.default_url).to eq('/preview.png')
+      end
     end
   end
 
@@ -308,9 +360,12 @@ describe CarrierWave::Uploader do
 
     describe '#store!' do
       before do
+        @uploader_class.cache_storage = :file
         @uploader_class.storage = mock_storage('base')
-        @uploader_class.version(:thumb).storage = mock_storage('thumb')
-        @uploader_class.version(:preview).storage = mock_storage('preview')
+        mock_thumb_storage = mock_storage('thumb')
+        @uploader_class.version(:thumb) { self.storage = mock_thumb_storage }
+        mock_preview_storage = mock_storage('preview')
+        @uploader_class.version(:preview) { self.storage = mock_preview_storage }
 
         @file = File.open(file_path('test.jpg'))
 
@@ -326,18 +381,21 @@ describe CarrierWave::Uploader do
         allow(@preview_stored_file).to receive(:path).and_return('/path/to/somewhere/preview')
         allow(@preview_stored_file).to receive(:url).and_return('http://www.example.com/preview')
 
-        @storage = double('a storage engine')
+        @storage = double('a storage instance')
         allow(@storage).to receive(:store!).and_return(@base_stored_file)
+        allow(@storage).to receive(:identifier).and_return('test.jpg')
 
-        @thumb_storage = double('a storage engine for thumbnails')
+        @thumb_storage = double('a storage instance for thumbnails')
         allow(@thumb_storage).to receive(:store!).and_return(@thumb_stored_file)
+        allow(@thumb_storage).to receive(:identifier).and_return('test.jpg')
 
-        @preview_storage = double('a storage engine for previews')
+        @preview_storage = double('a storage instance for previews')
         allow(@preview_storage).to receive(:store!).and_return(@preview_stored_file)
+        allow(@preview_storage).to receive(:identifier).and_return('test.jpg')
 
         allow(@uploader_class.storage).to receive(:new).with(@uploader).and_return(@storage)
-        allow(@uploader_class.version(:thumb).storage).to receive(:new).and_return(@thumb_storage)
-        allow(@uploader_class.version(:preview).storage).to receive(:new).and_return(@preview_storage)
+        allow(@uploader.thumb.class.storage).to receive(:new).and_return(@thumb_storage)
+        allow(@uploader.preview.class.storage).to receive(:new).and_return(@preview_storage)
       end
 
       it "should set the current path for the version" do
@@ -366,36 +424,72 @@ describe CarrierWave::Uploader do
         @uploader.store!
       end
 
-      it "should process conditional versions if the condition method returns true" do
-        @uploader_class.version(:preview).version_options[:if] = :true?
-        expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
-        @uploader.store!(@file)
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_present
+      context "when there is an 'if' option" do
+        it "should process conditional versions if the condition method returns true" do
+          @uploader_class.version(:preview, if: :true?)
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
+
+        it "should not process conditional versions if the condition method returns false" do
+          @uploader_class.version(:preview, if: :false?)
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
+
+        it "should process conditional version if the condition block returns true" do
+          @uploader_class.version(:preview, if: lambda{|record, args| record.true?(args[:file])})
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
+
+        it "should not process conditional versions if the condition block returns false" do
+          @uploader_class.version(:preview, if: lambda{|record, args| record.false?(args[:file])})
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
       end
 
-      it "should not process conditional versions if the condition method returns false" do
-        @uploader_class.version(:preview).version_options[:if] = :false?
-        expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
-        @uploader.store!(@file)
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_blank
-      end
+      context "when there is an 'unless' option" do
+        it "should not process conditional versions if the condition method returns true" do
+          @uploader_class.version(:preview, unless: :true?)
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
 
-      it "should process conditional version if the condition block returns true" do
-        @uploader_class.version(:preview).version_options[:if] = lambda{|record, args| record.true?(args[:file])}
-        expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
-        @uploader.store!(@file)
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_present
-      end
+        it "should process conditional versions if the condition method returns false" do
+          @uploader_class.version(:preview, unless: :false?)
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
 
-      it "should not process conditional versions if the condition block returns false" do
-        @uploader_class.version(:preview).version_options[:if] = lambda{|record, args| record.false?(args[:file])}
-        expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
-        @uploader.store!(@file)
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_blank
+        it "should not process conditional version if the condition block returns true" do
+          @uploader_class.version(:preview, unless: lambda{|record, args| record.true?(args[:file])})
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
+
+        it "should process conditional versions if the condition block returns false" do
+          @uploader_class.version(:preview, unless: lambda{|record, args| record.false?(args[:file])})
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.store!(@file)
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
       end
 
       it "should not cache file twice when store! called with a file" do
@@ -468,15 +562,53 @@ describe CarrierWave::Uploader do
         expect(File.exist?(@uploader.mini.path)).to eq(false)
       end
 
-      it "should not create version if proc returns false" do
-        @uploader_class.version(:mini, :if => Proc.new { |*args| false } )
+      it "should not process the file multiple times" do
+        file_size = @file.read.size
+        @uploader_class.class_eval do
+          process :shorten
+
+          def shorten
+            File.write(file.path, file.read[0...-1])
+          end
+        end
+
         @uploader.store!(@file)
+        @uploader.recreate_versions!
+        expect(@uploader.read.size + 1).to eq(file_size)
+        @uploader.recreate_versions!(:thumb)
+        expect(@uploader.read.size + 1).to eq(file_size)
+      end
 
-        expect(@uploader.mini.path).to be_nil
+      it "should not leave the cache_id set" do
+        @uploader.store!(@file)
+        @uploader.recreate_versions!
+        expect(@uploader).not_to be_cached
+      end
 
-        @uploader.recreate_versions!(:mini)
+      context "when there is an 'if' option" do
+        it "should not create version if proc returns false" do
+          @uploader_class.version(:mini, :if => Proc.new { |*args| false } )
+          @uploader.store!(@file)
 
-        expect(@uploader.mini.path).to be_nil
+          expect(@uploader.mini.path).to be_nil
+
+          @uploader.recreate_versions!(:mini)
+
+          expect(@uploader.mini.path).to be_nil
+        end
+      end
+
+      context "when there is an 'unless' option" do
+        it "should not create version if proc returns true" do
+          @uploader_class.version(:mini, :unless => Proc.new { |*args| true } )
+          @uploader.store!(@file)
+
+          expect(@uploader.mini.path).to be_nil
+
+          @uploader.recreate_versions!(:mini)
+
+          expect(@uploader.mini.path).to be_nil
+        end
       end
 
       it "should not change the case of versions" do
@@ -490,8 +622,10 @@ describe CarrierWave::Uploader do
 
     describe '#remove!' do
       before do
+        @uploader_class.cache_storage = :file
         @uploader_class.storage = mock_storage('base')
-        @uploader_class.version(:thumb).storage = mock_storage('thumb')
+        mock_thumb_storage = mock_storage('thumb')
+        @uploader_class.version(:thumb) { self.storage = mock_thumb_storage }
 
         @file = File.open(file_path('test.jpg'))
 
@@ -500,12 +634,14 @@ describe CarrierWave::Uploader do
 
         @storage = double('a storage engine')
         allow(@storage).to receive(:store!).and_return(@base_stored_file)
+        allow(@storage).to receive(:identifier).and_return('test.jpg')
 
         @thumb_storage = double('a storage engine for thumbnails')
         allow(@thumb_storage).to receive(:store!).and_return(@thumb_stored_file)
+        allow(@thumb_storage).to receive(:identifier).and_return('test.jpg')
 
         allow(@uploader_class.storage).to receive(:new).with(@uploader).and_return(@storage)
-        allow(@uploader_class.version(:thumb).storage).to receive(:new).with(@uploader.thumb).and_return(@thumb_storage)
+        allow(@uploader.thumb.class.storage).to receive(:new).with(@uploader.thumb).and_return(@thumb_storage)
 
         allow(@base_stored_file).to receive(:delete)
         allow(@thumb_stored_file).to receive(:delete)
@@ -536,8 +672,10 @@ describe CarrierWave::Uploader do
     describe '#retrieve_from_store!' do
       before do
         @uploader_class.storage = mock_storage('base')
-        @uploader_class.version(:thumb).storage = mock_storage('thumb')
-        @uploader_class.version(:preview).storage = mock_storage('preview')
+        mock_thumb_storage = mock_storage('thumb')
+        @uploader_class.version(:thumb) { self.storage = mock_thumb_storage }
+        mock_preview_storage = mock_storage('preview')
+        @uploader_class.version(:preview) { self.storage = mock_preview_storage }
 
         @file = File.open(file_path('test.jpg'))
 
@@ -563,8 +701,8 @@ describe CarrierWave::Uploader do
         allow(@preview_storage).to receive(:retrieve!).and_return(@preview_stored_file)
 
         allow(@uploader_class.storage).to receive(:new).with(@uploader).and_return(@storage)
-        allow(@uploader_class.version(:thumb).storage).to receive(:new).with(@uploader.thumb).and_return(@thumb_storage)
-        allow(@uploader_class.version(:preview).storage).to receive(:new).with(@uploader.preview).and_return(@preview_storage)
+        allow(@uploader.thumb.class.storage).to receive(:new).with(@uploader.thumb).and_return(@thumb_storage)
+        allow(@uploader.preview.class.storage).to receive(:new).with(@uploader.preview).and_return(@preview_storage)
       end
 
       it "should set the current path" do
@@ -592,20 +730,40 @@ describe CarrierWave::Uploader do
         expect(@uploader.filename).to be_nil
       end
 
-      it "should process conditional versions if the condition method returns true" do
-        @uploader_class.version(:preview).version_options[:if] = :true?
-        expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
-        @uploader.retrieve_from_store!('monkey.txt')
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_present
+      context "when there is an 'if' option" do
+        it "should process conditional versions if the condition method returns true" do
+          @uploader_class.version(:preview, if: :true?)
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.retrieve_from_store!('monkey.txt')
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
+
+        it "should not process conditional versions if the condition method returns false" do
+          @uploader_class.version(:preview, if: :false?)
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.retrieve_from_store!('monkey.txt')
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
       end
 
-      it "should not process conditional versions if the condition method returns false" do
-        @uploader_class.version(:preview).version_options[:if] = :false?
-        expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
-        @uploader.retrieve_from_store!('monkey.txt')
-        expect(@uploader.thumb).to be_present
-        expect(@uploader.preview).to be_blank
+      context "when there is an 'unless' option" do
+        it "should not process conditional versions if the condition method returns true" do
+          @uploader_class.version(:preview, unless: :true?)
+          expect(@uploader).to receive(:true?).at_least(:once).and_return(true)
+          @uploader.retrieve_from_store!('monkey.txt')
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_blank
+        end
+
+        it "should process conditional versions if the condition method returns false" do
+          @uploader_class.version(:preview, unless: :false?)
+          expect(@uploader).to receive(:false?).at_least(:once).and_return(false)
+          @uploader.retrieve_from_store!('monkey.txt')
+          expect(@uploader.thumb).to be_present
+          expect(@uploader.preview).to be_present
+        end
       end
     end
   end
@@ -637,6 +795,213 @@ describe CarrierWave::Uploader do
         expect(File.read(public_path(@uploader.to_s))).not_to eq(File.read(public_path(@uploader.thumb.to_s)))
         expect(File.read(public_path(@uploader.thumb.to_s))).to eq(File.read(public_path(@uploader.small_thumb.to_s)))
       end
+
+      it "should not cache an inactive version" do
+        @uploader_class.class_eval do
+          def condition(_); false; end
+        end
+        @uploader_class.version(:conditional_thumb, :from_version => :thumb, :if => :condition)
+
+        @uploader.cache!(File.open(file_path('bork.txt')))
+        expect(@uploader.conditional_thumb.cached?).to be false
+      end
+    end
+
+    describe "#recreate_versions!" do
+      let(:bork_file) { File.open(file_path('bork.txt')) }
+      let(:original_contents) { File.read(public_path(@uploader.to_s)) }
+      let(:thumb_contents) { File.read(public_path(@uploader.thumb.to_s)) }
+      let(:small_thumb_contents) { File.read(public_path(@uploader.small_thumb.to_s)) }
+
+      context "when the file is not stored" do
+        it "should not break" do
+          @uploader.recreate_versions!
+          @uploader.recreate_versions!(:small_thumb)
+        end
+      end
+
+      context "when no versions are given" do
+        it "should process file based on the version" do
+          @uploader.store!(bork_file)
+          @uploader.recreate_versions!
+          expect(thumb_contents).to eq(small_thumb_contents)
+        end
+      end
+
+      context "when version is given" do
+        it "should process file based on the version" do
+          @uploader.store!(bork_file)
+          FileUtils.rm([@uploader.small_thumb.path, @uploader.thumb.path])
+          @uploader.recreate_versions!(:small_thumb)
+          expect(File.exist?(public_path(@uploader.thumb.to_s))).to be true
+          expect(small_thumb_contents).to eq(thumb_contents)
+          expect(small_thumb_contents).not_to eq(original_contents)
+        end
+
+        it "reprocess parent version, too" do
+          @uploader.store!(bork_file)
+          FileUtils.rm(@uploader.thumb.path)
+          @uploader.recreate_versions!(:small_thumb)
+        end
+
+        it "works fine when recreating both dependent and parent versions" do
+          @uploader.store!(bork_file)
+          FileUtils.rm([@uploader.small_thumb.path, @uploader.thumb.path])
+          @uploader.recreate_versions!(:small_thumb, :thumb)
+          expect(File.exist?(public_path(@uploader.small_thumb.to_s))).to be true
+          expect(File.exist?(public_path(@uploader.thumb.to_s))).to be true
+
+          # doesn't depend on arguments order
+          FileUtils.rm([@uploader.small_thumb.path, @uploader.thumb.path])
+          @uploader.recreate_versions!(:thumb, :small_thumb)
+          expect(File.exist?(public_path(@uploader.small_thumb.to_s))).to be true
+          expect(File.exist?(public_path(@uploader.thumb.to_s))).to be true
+        end
+
+        it "doesn't touch other versions" do
+          @uploader_class.version(:another)
+          @uploader.store!(bork_file)
+          FileUtils.rm(@uploader.another.path)
+          @uploader.recreate_versions!(:small_thumb)
+          expect(File.exist?(public_path(@uploader.another.to_s))).to be false
+        end
+      end
+
+      context "with a grandchild version" do
+        it "should process all the files needed for recreation" do
+          @uploader_class.version(:grandchild, from_version: :small_thumb)
+          @uploader.store!(bork_file)
+          FileUtils.rm([@uploader.small_thumb.path, @uploader.thumb.path])
+          @uploader.recreate_versions!(:grandchild)
+          expect(File.exist?(public_path(@uploader.thumb.to_s))).to be true
+          expect(File.exist?(public_path(@uploader.small_thumb.to_s))).to be true
+          expect(File.exist?(public_path(@uploader.grandchild.to_s))).to be true
+        end
+      end
+    end
+  end
+
+  describe 'with a version using #convert' do
+    before do
+      @uploader_class.class_eval do
+        include CarrierWave::MiniMagick
+      end
+
+      @uploader_class.version(:thumb) do
+        process convert: :png
+      end
+
+      @another_uploader = @uploader_class.new
+    end
+
+    it 'caches the file with given extension' do
+      @uploader.cache!(File.open(file_path('landscape.jpg')))
+      expect(File.basename(@uploader.thumb.cache_path)).to eq 'thumb_landscape.png'
+      expect(File.basename(@uploader.thumb.url)).to eq 'thumb_landscape.png'
+      expect(@uploader.thumb).to be_format('png')
+    end
+
+    it 'retrieves the cached file' do
+      @uploader.cache!(File.open(file_path('landscape.jpg')))
+      @another_uploader.retrieve_from_cache!(@uploader.cache_name)
+      expect(@another_uploader.url).to eq @uploader.url
+      expect(@another_uploader.thumb.url).to eq @uploader.thumb.url
+    end
+
+    it 'stores the file with given extension' do
+      @uploader.store!(File.open(file_path('landscape.jpg')))
+      expect(File.basename(@uploader.thumb.store_path)).to eq 'thumb_landscape.png'
+      expect(File.basename(@uploader.thumb.url)).to eq 'thumb_landscape.png'
+      expect(@uploader.thumb).to be_format('png')
+    end
+
+    it 'retrieves the stored file' do
+      @uploader.store!(File.open(file_path('landscape.jpg')))
+      @another_uploader.retrieve_from_store!(@uploader.identifier)
+      expect(@another_uploader.identifier).to eq @uploader.identifier
+      expect(@another_uploader.url).to eq @uploader.url
+      expect(@another_uploader.thumb.url).to eq @uploader.thumb.url
+    end
+
+    context 'with base filename overridden' do
+      before do
+        @uploader_class.class_eval do
+          def filename
+            "image.#{file.extension}"
+          end
+        end
+      end
+
+      it "stores the file" do
+        @uploader.store!(File.open(file_path('landscape.jpg')))
+        expect(File.basename(@uploader.thumb.url)).to eq 'thumb_image.png'
+      end
+
+      it "retrieves the file without inconsistency" do
+        @uploader.store!(File.open(file_path('landscape.jpg')))
+        @another_uploader.retrieve_from_store!(@uploader.identifier)
+        expect(@another_uploader.identifier).to eq @uploader.identifier
+        expect(@another_uploader.url).to eq @uploader.url
+        expect(@another_uploader.thumb.url).to eq @uploader.thumb.url
+      end
+    end
+
+    context 'with version full_filename overridden' do
+      before do
+        @uploader_class.version(:thumb) do
+          def full_filename(for_file)
+            'thumb_image.png'
+          end
+        end
+      end
+
+      it "stores the file" do
+        @uploader.store!(File.open(file_path('landscape.jpg')))
+        expect(File.basename(@uploader.thumb.url)).to eq 'thumb_image.png'
+      end
+
+      it "retrieves the file without inconsistency" do
+        @uploader.store!(File.open(file_path('landscape.jpg')))
+        @another_uploader.retrieve_from_store!(@uploader.identifier)
+        expect(@another_uploader.identifier).to eq @uploader.identifier
+        expect(@another_uploader.url).to eq @uploader.url
+        expect(@another_uploader.thumb.url).to eq @uploader.thumb.url
+      end
+    end
+  end
+
+  describe 'with a version with file extension change' do
+    before do
+      @uploader_class.class_eval do
+        def rename_to_bin
+          file.move_to("#{File.basename(file.filename, '.*')}.bin")
+        end
+      end
+
+      @uploader_class.version(:thumb) do
+        process :rename_to_bin
+      end
+    end
+
+    it "does not change the version's #filename" do
+      @uploader.cache!(File.open(file_path('landscape.jpg')))
+      expect(@uploader.thumb.file.filename).to eq 'landscape.bin'
+      expect(@uploader.thumb.filename).to eq 'landscape.jpg'
+      expect(File.basename(@uploader.thumb.store_path)).to eq 'thumb_landscape.jpg'
+    end
+
+    context "but applying #force_extension" do
+      before do
+        @uploader_class.version(:thumb) do
+          force_extension '.bin'
+        end
+      end
+
+      it "changes #filename to have the extension" do
+        @uploader.store!(File.open(file_path('landscape.jpg')))
+        expect(@uploader.thumb.identifier).to eq 'landscape.jpg'
+        expect(File.basename(@uploader.thumb.store_path)).to eq 'thumb_landscape.bin'
+      end
     end
   end
 end
